@@ -26,12 +26,17 @@
 # ------------------------------------------------------------------------------
 
 import re
-from datetime import timedelta, date
-from dateparser import parse as parse_datetime
+from collections.abc import Mapping
+from datetime import date, datetime, timedelta
+
+from dateparser import parse as _parse_datetime
 
 __all__ = [
-    'parse_datetime', 'RE_ISO_8601', 'parse_duration',
-    'like_pattern_to_re_pattern', 'like_pattern_to_re'
+    "parse_datetime",
+    "RE_ISO_8601",
+    "parse_duration",
+    "like_pattern_to_re_pattern",
+    "like_pattern_to_re",
 ]
 
 RE_ISO_8601 = re.compile(
@@ -46,41 +51,41 @@ RE_ISO_8601 = re.compile(
 
 
 def parse_duration(value: str) -> timedelta:
-    """ Parses an ISO 8601 duration string into a python timedelta object.
-        Raises a ``ValueError`` if a conversion was not possible.
+    """Parses an ISO 8601 duration string into a python timedelta object.
+    Raises a ``ValueError`` if a conversion was not possible.
 
-        :param value: the ISO8601 duration string to parse
-        :type value: str
-        :return: the parsed duration
-        :rtype: datetime.timedelta
+    :param value: the ISO8601 duration string to parse
+    :type value: str
+    :return: the parsed duration
+    :rtype: datetime.timedelta
     """
 
     match = RE_ISO_8601.match(value)
     if not match:
-        raise ValueError(
-            "Could not parse ISO 8601 duration from '%s'." % value
-        )
-    match = match.groupdict()
+        raise ValueError("Could not parse ISO 8601 duration from '%s'." % value)
+    parts = match.groupdict()
 
-    sign = -1 if "-" == match['sign'] else 1
-    days = float(match['days'] or 0)
-    days += float(match['months'] or 0) * 30  # ?!
-    days += float(match['years'] or 0) * 365  # ?!
-    fsec = float(match['seconds'] or 0)
-    fsec += float(match['minutes'] or 0) * 60
-    fsec += float(match['hours'] or 0) * 3600
+    sign = -1 if "-" == parts["sign"] else 1
+    days = float(parts["days"] or 0)
+    days += float(parts["months"] or 0) * 30  # ?!
+    days += float(parts["years"] or 0) * 365  # ?!
+    fsec = float(parts["seconds"] or 0)
+    fsec += float(parts["minutes"] or 0) * 60
+    fsec += float(parts["hours"] or 0) * 3600
 
     return sign * timedelta(days, fsec)
 
 
 def parse_date(value: str) -> date:
-    """ Backport for `fromisoformat` for dates in Python 3.6
-    """
+    """Backport for `fromisoformat` for dates in Python 3.6"""
+    return date(*(int(part) for part in value.split("-")))
 
-    if hasattr(date, 'fromisoformat'):
-        return date.fromisoformat(value)
 
-    return date(*(int(part) for part in value.split('-')))
+def parse_datetime(value: str) -> datetime:
+    parsed = _parse_datetime(value)
+    if parsed is None:
+        raise ValueError(value)
+    return parsed
 
 
 def like_pattern_to_re_pattern(like, wildcard, single_char, escape_char):
@@ -91,8 +96,8 @@ def like_pattern_to_re_pattern(like, wildcard, single_char, escape_char):
     dx_single_char = re.escape(x_single_char)
 
     # special handling if escape char clashes with re escape char
-    if escape_char == '\\':
-        x_escape_char = '\\\\\\\\'
+    if escape_char == "\\":
+        x_escape_char = "\\\\\\\\"
     else:
         x_escape_char = re.escape(escape_char)
     dx_escape_char = re.escape(x_escape_char)
@@ -101,39 +106,52 @@ def like_pattern_to_re_pattern(like, wildcard, single_char, escape_char):
 
     # handle not escaped wildcards/single chars
     pattern = re.sub(
-        f'(?<!{x_escape_char}){dx_wildcard}',
-        '.*',
+        f"(?<!{x_escape_char}){dx_wildcard}",
+        ".*",
         pattern,
     )
     pattern = re.sub(
-        f'(?<!{x_escape_char}){dx_single_char}',
-        '.',
+        f"(?<!{x_escape_char}){dx_single_char}",
+        ".",
         pattern,
     )
 
     # handle escaped wildcard, single chars and escape chars
     pattern = re.sub(
-        f'{dx_escape_char}{dx_wildcard}',
+        f"{dx_escape_char}{dx_wildcard}",
         x_wildcard,
         pattern,
     )
     pattern = re.sub(
-        f'{dx_escape_char}{dx_single_char}',
+        f"{dx_escape_char}{dx_single_char}",
         x_single_char,
         pattern,
     )
     pattern = re.sub(
-        f'{x_escape_char}{x_escape_char}',
+        f"{x_escape_char}{x_escape_char}",
         x_escape_char,
         pattern,
     )
 
-    return f'^{pattern}$'
+    return f"^{pattern}$"
 
 
 def like_pattern_to_re(like, nocase, wildcard, single_char, escape_char):
     flags = re.I if nocase else 0
     return re.compile(
         like_pattern_to_re_pattern(like, wildcard, single_char, escape_char),
-        flags=flags
+        flags=flags,
     )
+
+
+class IdempotentDict(Mapping):
+    "A dict class that always returns the key"
+
+    def __getitem__(self, key):
+        return key
+
+    def __iter__(self):
+        return iter(())
+
+    def __len__(self) -> int:
+        return 0
